package com.qmth.ops.biz.ai.client;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.boot.core.ai.model.llm.endpoint.ChatEndpoint;
import com.qmth.boot.core.rateLimit.service.RateLimiter;
import com.qmth.boot.core.rateLimit.service.impl.MemoryRateLimiter;
import com.qmth.ops.biz.ai.exception.ChatRateLimitExceeded;
import okhttp3.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.Duration;

/**
 * 大模型chat类接口基础实现
 */
public abstract class ChatApiClient {

    private static final Logger log = LoggerFactory.getLogger(ChatApiClient.class);

    private OkHttpClient client;

    private ObjectMapper mapper;

    private ChatApiConfig config;

    private RateLimiter queryRateLimiter;

    public ChatApiClient(ChatApiConfig config) {
        this.client = new OkHttpClient.Builder().connectionPool(new ConnectionPool())
                .connectTimeout(Duration.ofSeconds(10)).readTimeout(Duration.ofSeconds(50)).build();
        this.mapper = new ObjectMapper();
        this.config = config;
        if (config.getQpm() > 0) {
            this.queryRateLimiter = new MemoryRateLimiter(config.getQpm(), 60 * 1000);
        }
    }

    protected ChatApiConfig getConfig() {
        return config;
    }

    public abstract ChatEndpoint buildEndpoint(ChatRequest request, LlmAppType appType);

    protected abstract Headers buildHeader(Headers.Builder headerBuilder, LlmAppType appType);

    protected abstract Object buildRequest(ChatRequest request, LlmAppType appType);

    protected abstract ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException;

    protected abstract ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper);

    public ChatResult call(ChatRequest request, LlmAppType appType) throws Exception {
        if (queryRateLimiter != null && !queryRateLimiter.acquire()) {
            throw new ChatRateLimitExceeded(config.getSupplier(), config.getModel(), config.getQpm());
        }

        Object params = buildRequest(request, appType);
        if (log.isDebugEnabled()) {
            log.debug("ChatRequest:{}", mapper.writeValueAsString(params));
        }

        RequestBody body = RequestBody
                .create(MediaType.parse("application/json"), mapper.writeValueAsBytes(params));
        Request httpRequest = new Request.Builder().url(config.getUrl())
                .headers(buildHeader(new Headers.Builder(), appType)).post(body).build();

        try (Response response = client.newCall(httpRequest).execute();) {
            byte[] data = response.body() != null ? response.body().bytes() : null;
            if (log.isDebugEnabled()) {
                log.debug("ChatResult:{}", data != null ? new String(data, StandardCharsets.UTF_8) : null);
            }

            if (response.isSuccessful()) {
                return buildResult(data, mapper);
            } else {
                return handleError(data, response.code(), mapper);
            }
        }
    }

}
