package com.qmth.ops.biz.ai.client;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.ocr.ImageType;
import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.boot.core.rateLimit.service.RateLimiter;
import com.qmth.boot.core.rateLimit.service.impl.MemoryRateLimiter;
import com.qmth.ops.biz.ai.exception.OcrRateLimitExceeded;
import okhttp3.*;

import java.io.IOException;
import java.time.Duration;

/**
 * 大模型chat类接口基础实现
 */
public abstract class OcrApiClient {

    private OkHttpClient client;

    private OcrApiConfig config;

    private ObjectMapper mapper;

    private RateLimiter queryRateLimiter;

    public OcrApiClient(OcrApiConfig config) {
        OkHttpClient.Builder buidler = new OkHttpClient.Builder().connectionPool(new ConnectionPool())
                .connectTimeout(Duration.ofSeconds(10)).readTimeout(Duration.ofSeconds(50));
        Interceptor interceptor = getInterceptor();
        if (interceptor != null) {
            buidler.addInterceptor(interceptor);
        }
        this.client = buidler.build();
        this.mapper = new ObjectMapper();
        this.config = config;
        if (config.getQps() > 0) {
            this.queryRateLimiter = new MemoryRateLimiter(config.getQps(), 1000);
        }
    }

    protected OcrApiConfig getConfig() {
        return config;
    }

    protected abstract Request buildRequest(OcrType type, byte[] image, ImageType imageType) throws Exception;

    protected abstract String buildResult(byte[] data, ObjectMapper mapper) throws IOException;

    protected abstract String handleError(byte[] data, int statusCode, ObjectMapper mapper);

    protected Interceptor getInterceptor() {
        return null;
    }

    public String forImage(OcrType type, byte[] image, ImageType imageType) throws Exception {
        if (queryRateLimiter != null && !queryRateLimiter.acquire()) {
            throw new OcrRateLimitExceeded(config.getQps());
        }

        try (Response response = client.newCall(buildRequest(type, image, imageType)).execute()) {
            byte[] data = response.body() != null ? response.body().bytes() : null;
            if (response.isSuccessful()) {
                return buildResult(data, mapper);
            } else {
                return handleError(data, response.code(), mapper);
            }
        }
    }

}
