package com.qmth.ops.biz.ai.client.aliyun.llm;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.ChatRole;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.boot.core.ai.model.llm.endpoint.ChatEndpoint;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.client.aliyun.AliyunError;
import com.qmth.ops.biz.ai.exception.ChatRequestError;
import okhttp3.Headers;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class AliyunChatClient extends ChatApiClient {

    private static final String AUTH_HEADER_NAME = "Authorization";

    private static final String AUTH_HEADER_VALUE = "Bearer ";

    public AliyunChatClient(ChatApiConfig config) {
        super(config);
    }

    @Override
    public ChatEndpoint buildEndpoint(ChatRequest request, LlmAppType appType) {
        ChatEndpoint endpoint = new ChatEndpoint();
        endpoint.setUrl(getConfig().getUrl());
        endpoint.setMethod("POST");

        Map<String, String> headers = new HashMap<>();
        headers.put(AUTH_HEADER_NAME, AUTH_HEADER_VALUE + getConfig().getSecret());
        if (request.isStream()) {
            headers.put("X-DashScope-SSE", "enable");
        }
        endpoint.setHeaders(headers);

        this.buildRequest(request, appType);

        byte[] jsonBytes;
        try {
            jsonBytes = new ObjectMapper().writeValueAsBytes(request);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
        endpoint.setRequestBody(jsonBytes);

        return endpoint;
    }

    @Override
    protected Headers buildHeader(Headers.Builder headerBuilder, LlmAppType appType) {
        headerBuilder.add(AUTH_HEADER_NAME, AUTH_HEADER_VALUE + getConfig().getSecret());
        return headerBuilder.build();
    }

    @Override
    protected Object buildRequest(ChatRequest request, LlmAppType appType) {
        request.setModel(getConfig().getModel());
        if (appType == LlmAppType.AUTO_SCORE) {
            request.setTopP(0.1f);
        } else if (appType == LlmAppType.AUTO_GENERATE_QUESTION) {
            request.setTopP(0.9f);
        }
        request.setResultFormat("message");
        return request;
    }

    @Override
    protected ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        return mapper.readValue(data, ChatResult.class);
    }

    @Override
    protected ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        AliyunError error = null;
        if (data != null) {
            try {
                error = mapper.readValue(data, AliyunError.class);
            } catch (Exception ignore) {
            }
        }
        switch (statusCode) {
            case 400:
                throw new ChatRequestError(error != null ? error.getMessage() : "chat request error");
            case 429:
                throw new ReentrantException(error != null ? error.getMessage() : "chat model rate limit exceeded");
            default:
                throw new StatusException(error != null ? error.getMessage() : "chat model error");
        }
    }

    public static void main(String[] args) throws Exception {
        ChatApiConfig config = new ChatApiConfig();
        config.setSupplier("Aliyun");
        config.setUrl("https://dashscope.aliyuncs.com/compatible-mode/v1/chat/completions");
        config.setSecret("sk-xxx");
        // config.setModel("qwen3-235b-a22b");
        // config.setModel("qwen-plus");
        // config.setModel("qwen-max");
        // config.setModel("qwen-turbo");
        // config.setModel("qwen-long");
        config.setModel("deepseek-v3");
        config.setQpm(60);
        AliyunChatClient client = new AliyunChatClient(config);
        ChatRequest request = new ChatRequest();
        request.addMessage(ChatRole.user,
                "作为小学数学科目的命题老师，请按照下列要求出1道单选试题，\n"
                        + "试题题干前用单独一行【题干】作为内容，\n"
                        + "试题答案前用单独一行【答案】作为内容，\n"
                        + "试题答案解析前用单独一行【解析】作为内容，\n"
                        + "试题包含4个选项，选项内容前用单独一行【选项】作为内容，且每个选项前用大写英文字母开头，\n"
                        + "请按照上述要求出1道小学数学的单选试题。");
        ChatResult chatResult = client.call(request, LlmAppType.AUTO_GENERATE_QUESTION);
        System.out.println(new ObjectMapper().writeValueAsString(chatResult));
    }

}
